#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>
#include <math.h>


int main(int argc, char** argv) {
    MPI_Init(&argc, &argv);

    int rank, size;
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    MPI_Comm_size(MPI_COMM_WORLD, &size);

    int vectorSize = 16; // Vector size
    int chunkSize = vectorSize / size; // Size of vector chunk for each process

    double *localVector = (double*)malloc(chunkSize * sizeof(double));
    double *globalVector = (double*)malloc(vectorSize * sizeof(double));

    // Process with rank 0 generates vector
    if (rank == 0) {
        for (int i = 0; i < vectorSize; i++) {
            globalVector[i] = 2;
        }
    }

    // Distribution of vector chunks over all processes
    MPI_Scatter(globalVector, chunkSize, MPI_DOUBLE, localVector, chunkSize, MPI_DOUBLE, 0, MPI_COMM_WORLD);

    // Local sum of squares
    double localSumOfSquares = 0.0;
    for (int i = 0; i < chunkSize; i++) {
        localSumOfSquares += localVector[i] * localVector[i];
    }

    // Summation of local sum of squares from processes into global sum of squares
    double globalSumOfSquares = 0.0;
    MPI_Allreduce(&localSumOfSquares, &globalSumOfSquares, 1, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
    
    // Square root of global sum of squares
    double globalNorm = sqrt(globalSumOfSquares);
    
    // Normalization of vector split into chunks on each process
    for (int i = 0; i < chunkSize; i++) {
        localVector[i] /= globalNorm;
    }

    // Collecting chunks into vector on process with rank 0
    MPI_Gather(localVector, chunkSize, MPI_DOUBLE, globalVector, chunkSize, MPI_DOUBLE, 0, MPI_COMM_WORLD);

    // Printing vector on process with rank 0
    if (rank == 0) {
        printf("Normalized Vector: {");
        for (int i = 0; i < vectorSize; i++) {
            printf("%.2f ", globalVector[i]);
        }
        printf("}\n");
    }

    MPI_Finalize();

    return 0;
}